#!@BIN_PERL@
#
#
#    Short description: Creates/Updates RRA archives, from nagios performance data
#    Copyright (C) 2006 - 2009 Badri Prasad Pillai
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
#
#    Contact details:
#               Badri(at)DigLinks.COM

#
$| = 1;    # disable buffering

use strict;
# to avoid Apache/mod_perl filling log file with redefines
no warnings qw(redefine);

# Variable declaration
use vars qw(
  $query_string   @key_value %service_map
  %info        %graph           $cache_dir
  $rrd_image_file   $rrd_file    @rrdtool_options @rrdtool_source
  $graph_template %config      @zoom_javascript $graph_height
  $graph_width    $total_graphs   $version      $footer
  $rrd_files      $tmp_rrd_files    $conf_file 	$data_flow      $debug
  $col            $g 		$RRD_ERROR %query_variables %query_variables_have
  $nagios_env_file $nagios_env $nagios_env_count $ds_menu @dsnames $dshash $now
  %colors @colors $perfdata $object $object_start $object_end $nagios_version
  $fact_bytes $fact_seconds *dir $rrd_dirnames $imgformat_defined

);

######################################################
# Change following variable if required
######################################################
$conf_file = "@CONF_DIR@/n2rrd.conf";

#
# commnet following line if you want to use configuration file
# to overide some default variables
#
#undef $conf_file;
undef $RRD_ERROR;

$debug = 1;

# for imgformat option check
$imgformat_defined=0;


#
# if $DATA_FLOW$ variable is defined in graph template
# then add the daily,weekly,monthely and yearly seconds value to calculate e.g. throughput
#
$data_flow = 0;

######################################################
# Normaly no need to change any things below this line
######################################################

$version = "@VERSION@";
$footer =
" <br><strong>Query URL: http://$ENV{SERVER_NAME}$ENV{SCRIPT_NAME}?$ENV{QUERY_STRING} </strong> <HR>Generated by <a href=\"http://n2rrd.diglinks.com\""
  . ">rrd2graph</a>($version), total graphs =";

# declare convertion values
$fact_bytes = {
   "KB" => {
      'val'     => 2**10,    #KiloBytes
      'result'  => "B",
      'xresult' => "Bytes"
   },
   "MB" => {
      'val'     => 2**20,    #MegaBytes
      'result'  => "B",
      'xresult' => "Bytes"
   },
   "GB" => {
      'val'     => 2**30,    #GigaBytes
      'result'  => "B",
      'xresult' => "Bytes"
   },
   "TB" => {
      'val'     => 2**40,    #TeraBytes
      'result'  => "B",
      'xresult' => "Bytes"
   },
   "PB" => {
      'val'     => 2**50,    #PetaBytes
      'result'  => "B",
      'xresult' => "Bytes"
   },
   "EB" => {
      'val'     => 2**60,    #ExaBytes
      'result'  => "B",
      'xresult' => "Bytes"
   },
};

$fact_seconds = {
   "MS" => {
      'val'     => 10**-3,     #millisecond
      'result'  => 's',
      'xresult' => 'seconds'
   },
   "US" => {
      'val'     => 10**-6,     #microsecond
      'result'  => 's',
      'xresult' => 'seconds'
   },
   "NS" => {
      'val'     => 10**-9,     #nanosecond
      'result'  => 's',
      'xresult' => 'seconds'
   }
};

$rrd_files        = {};
%service_map      = ();
$tmp_rrd_files    = {};
%info             = ();
@rrdtool_options  = ();
$total_graphs     = 0;
$nagios_env_count = 0;
%colors           = ();
@colors           = ();
$perfdata         = {};
$object           = {};
$object_start     = "";
$object_end       = "";

#
$now = time;

sub wanted;
sub create_thumb ($$$$$);
sub display_host ($$);
sub summary($);

sub log_msg($);
sub zoom ($$);
sub load_config($);
sub load_nagios_env($);
sub list_default_values();
sub random_string($$);
sub gen_graph($$$$$);

#
# If no configuration file given, then use default values
#
%config = (
   CONF_DIR                     => "@CONF_DIR@",
   TEMPLATES_DIR                => "@TEMPLATES_DIR@",
   SERVICE_NAME_MAPS            => "@TEMPLATES_DIR@/maps/service_name_maps",
   ZOOM_JS                      => "js/zoom.js",
   RRA_DIR                      => "@RRA_DIR@",
   LOGFILE                      => "@LOGFILE@",
   DOCUMENT_ROOT                => "@DOCUMENT_ROOT@",
   CACHE_DIR                    => "@CACHE_DIR@",
   CGIBIN                       => "@CGIBIN@",
   NAGIOS_CGIBIN                => "nagios/cgi-bin",
   RRDTOOL                      => "@RRDTOOL@",
   RRD_PATH_HIDDEN              => 0,
   NAGIOS_HOST_URL              => 1,
   TMPDIR                       => "@TMPDIR@",
   THUMB_WIDTH                  => 200,
   THUMB_HEIGHT                 => 100,
   THUMB_DISPLAY                => "Daily",
   THUMB_DISPLAY_COLUMNS        => 3,
   SAVE_NAGIOS_ENVIRONMENT_VARS => 0,
   TEMPLATE_NAME_AT_BEGINNING   => 0,
   DEMO_STR_LENGTH              => 3,
   PREDICTION_Daily             => "+2d",
   PREDICTION_Weekly            => "+7d",
   PREDICTION_Monthly           => "+15d",
   PREDICTION_Yearly            => "+6m",
   DYN_RRA_CREATE               => 0,
   DYN_GRAPH_CREATE             => 0,
   DYN_ADD_NEW_DS               => 0,
   DYN_RRA_PLUGIN_MAPS          => "@TEMPLATES_DIR@/maps/rra_plugin_maps",
   DYN_RRA_XFF                  => 0.5,
   DYN_CF                       => "AVERAGE,MAX,MIN,LAST",
   DYN_ARCHIVE_STEPS_DAYS       => "1:2,30:32,120:189,1440:1890",
   DYN_DS_CONVERT_TO_BYTES      => 1,
   DYN_DS_CONVERT_TO_SECONDS    => 0,
   DYN_RGB_COLORS_MAPS          => "@TEMPLATES_DIR@/maps/rgb.txt",
   DYN_GRAPH_USED_VALUE_COLOR   => '33a3ff',
   NAGIOS_STATUS_FILE           => '@NAGIOS_STATUS_FILE@',
   OPTIONAL_IMAGE_FORMATS 	=> 0,
   OPTIONAL_CGIBIN2 		=> "cgi-bin",
);


%query_variables = (
   service         => "",
   match_host      => "",
   summary         => "",
   hostname        => "",
   groupname   	   => "",
   action          => "",
   display         => "",
   graph_height    => "",
   graph_width     => "",
   graph_start     => "",
   graph_end       => "",
   title_font_size => "",
   el              => "",
   source          => "",
   extmacro        => "",
   extvalue        => "",
   demo            => "",
   ds_name         => "",
   dissect         => "",
   image_format    => "",
   _t              => "" # NagVis passes it, so don't ignore it

);

%query_variables_have = ();

load_config($conf_file) if $conf_file;

#
# Abort if query string is not defined
log_msg "Environment Variable QUERY_STRING undefined\n"
  and list_default_values
  if not $ENV{'QUERY_STRING'};

#
# set to rrd default
$graph_width  = 400;
$graph_height = 100;

@key_value = ();

%graph = (
   Daily           => "--start=-1d",
   Daily_seconds   => 86400,
   Weekly          => '--start=-604800',
   Weekly_seconds  => 604800,
   Monthly         => '--start=-1m',
   Monthly_seconds => 2592000,
   Yearly          => '--start=-1y',
   Yearly_seconds  => 31536000
);

$query_string = $ENV{'QUERY_STRING'};

# save key value pair
@key_value = split( /&/, $query_string );

#
# evaluate each value
foreach my $kv (@key_value) {

   # MIME code cleaning
   $kv =~ tr/+/ /;

   # MIME 8-bit hex  value convertion
   $kv =~ s/%([a-fA-F0-9].)/pack("C", hex($1))/eg;

   # XSSI clean !!
   $kv =~ s/<!--(.|\n)*-->//g;

   # Remove unwanted characters
   $kv =~ s/[^a-zA-Z0-9\-_=\.:\ ]//g;

   my ( $key, $value ) = split( /=/, $kv );

   $config{SAVE_NAGIOS_ENVIRONMENT_VARS} = $value if $key eq "extended_info";
   next if $key eq "extended_info";
   next if $key eq "ds_name" and $value eq "undefined";
   next if $key eq "groupname" and $value eq "undefined";

   #
   # Validate variable name
   #
   if ( defined $query_variables{$key} ) {
      if ( length($value) > 0 ) {
	 if ( defined $query_variables_have{$key} ) {
	    log_msg "Query string variable name \"$key\" already defined\n";
	    exit 1;
	 }
	 else {
	    $info{$key} = $value;
	    $query_variables_have{$key}++;
	 }
      }
      else {

	 #log_msg "Missing value for query string variable name \"$key\"\n";
	 next;
      }
   }
   else {
      log_msg "Unknown query string variable name \"$key\"\n";
      exit 1;
   }

   #print "$key = $value\n";
}

#
# assume there are no nagios variables if NAGIOS_URL switch is disabled
#
$config{SAVE_NAGIOS_ENVIRONMENT_VARS} = 0 if not $config{NAGIOS_HOST_URL};

#
# Store service mapping in a hash
#
      open S_MAPS, "< $config{CONF_DIR}/$config{SERVICE_NAME_MAPS}"
	or log_msg
"Can't open service maps file \"$config{CONF_DIR}/$config{SERVICE_NAME_MAPS}\"\n"
	and exit 1;

      log_msg
"Searching map in file \"$config{CONF_DIR}/$config{SERVICE_NAME_MAPS}\" for service \"$info{service}\" \n"
	if $debug;

while (<S_MAPS>) {
	next if /^#/;    # Skip comments
	next if /^$/;    # Skip empty lines
	s/#.*//;         # Remove partial comments
	chomp;

	$service_map{$1} = $2 if (/^\s*(.*):\s*(.*)/);
}
#

if ( !defined $info{summary} ) {
   @rrdtool_source  = ();
   @rrdtool_options = ();

   #
   # Sanity check: if RRA directory exists
   log_msg "Direcotry rra_dir = \"$config{CONF_DIR}\" doesn't exists\n"
     and exit 1
     if !-d $config{CONF_DIR};

   #
   # Sanity check: if templates directory exists
   log_msg
"Directory templates_dir = \"$config{CONF_DIR}/$config{TEMPLATES_DIR}\" doesn't exists\n"
     and exit 1
     if !-d "$config{CONF_DIR}/$config{TEMPLATES_DIR}";

   #
   # save original service name
   $info{service_orig} = $info{service};

   #
   # assume delimiter between service name is '_'
   my @parse_service_str = split /_/, $info{service};

   if ( $#parse_service_str <= 0 ) {

      foreach my $key ( keys(%service_map)) {
        $info{service} = $service_map{$key} if ($key eq $info{service}) ;
      }
  }
   else {
      if ( $config{TEMPLATE_NAME_AT_BEGINNING} ) {

	 # The first word after terminator character "_"
	 $info{service} = $parse_service_str[0];
      }
      else {

	 # The last word after terminator character "_"
	 $info{service} = $parse_service_str[$#parse_service_str];
      }
   }

   #
   # from config info rrd_file is the following
   $rrd_file =
       "$config{RRA_DIR}/"
     . ( defined $info{groupname} ? $info{groupname} : $info{hostname} )
     . "/$info{hostname}_$info{service}.rrd";

   #
   # check order same as rra template
   # 1. hostname_service.t
   # 2. service.t
   # 3. as per rewrite info
   if ( defined( $info{ds_name} ) ) {
      if (
	 -f "$config{CONF_DIR}/$config{TEMPLATES_DIR}/graph/$info{hostname}_$info{service}_$info{ds_name}.t"
	)
      {
	 $graph_template =
"$config{CONF_DIR}/$config{TEMPLATES_DIR}/graph/$info{hostname}_$info{service}_$info{ds_name}.t";
      }
      elsif (
	 -f "$config{CONF_DIR}/$config{TEMPLATES_DIR}/graph/$info{service}_$info{ds_name}.t"
	)
      {
	 $graph_template =
"$config{CONF_DIR}/$config{TEMPLATES_DIR}/graph/$info{service}_$info{ds_name}.t";
      }
      elsif (
	 -f "$config{CONF_DIR}/$config{TEMPLATES_DIR}/graph/dyn/$info{hostname}/$info{service}_$info{ds_name}.t"
	)
      {
	 $graph_template =
"$config{CONF_DIR}/$config{TEMPLATES_DIR}/graph/dyn/$info{hostname}/$info{hostname}_$info{service}_$info{ds_name}.t";
      }
      elsif ( $config{DYN_GRAPH_CREATE} )
      {    # create template dynamically if setup to do so

	 $graph_template =
"$config{CONF_DIR}/$config{TEMPLATES_DIR}/graph/dyn/$info{hostname}/$info{hostname}_$info{service}_$info{ds_name}.t";


	 if ( !-f $graph_template ) {    # create template if doesn't exist

	   log_msg "Creating graph template \"$graph_template\" dynamically \n" if $debug;

	    mkdir "$config{CONF_DIR}/$config{TEMPLATES_DIR}/graph/dyn"
	      if !-d "$config{CONF_DIR}/$config{TEMPLATES_DIR}/graph/dyn";

	    mkdir
"$config{CONF_DIR}/$config{TEMPLATES_DIR}/graph/dyn/$info{hostname}"
	      if
	       !-d "$config{CONF_DIR}/$config{TEMPLATES_DIR}/graph/dyn/$info{hostname}";

	    my $check_interval =
	      &get_check_interval( $info{hostname}, "SERVICE",
	       $info{service_orig}, $config{NAGIOS_STATUS_FILE} );
	    if ($check_interval) {
	       &parse_perfdata( $object->{service}->{performance_data} );
	       &gen_graph_template( $info{ds_name} );
	       &write_graph_template_to_file("$graph_template");
	    }
	    else {
		log_msg "Couldn't get check_interval \"$check_interval\" for  service \"$info{service_orig}\" \n" if $debug;
	    }
	 }
      }
   }
   else {
      if (
	 -f "$config{CONF_DIR}/$config{TEMPLATES_DIR}/graph/$info{hostname}_$info{service}.t"
	)
      {
	 $graph_template =
"$config{CONF_DIR}/$config{TEMPLATES_DIR}/graph/$info{hostname}_$info{service}.t";
      }
      elsif (
	 -f "$config{CONF_DIR}/$config{TEMPLATES_DIR}/graph/$info{service}.t" )
      {
	 $graph_template =
	   "$config{CONF_DIR}/$config{TEMPLATES_DIR}/graph/$info{service}.t";
      }
      elsif ( $config{DYN_GRAPH_CREATE} ) {

	 $graph_template =
"$config{CONF_DIR}/$config{TEMPLATES_DIR}/graph/dyn/$info{hostname}/$info{hostname}_$info{service}.t";
	 if ( !-f $graph_template ) {
	    mkdir "$config{CONF_DIR}/$config{TEMPLATES_DIR}/graph/dyn"
	      if !-d "$config{CONF_DIR}/$config{TEMPLATES_DIR}/graph/dyn";

	    mkdir
"$config{CONF_DIR}/$config{TEMPLATES_DIR}/graph/dyn/$info{hostname}"
	      if
	       !-d "$config{CONF_DIR}/$config{TEMPLATES_DIR}/graph/dyn/$info{hostname}";

	    my $check_interval =
	      &get_check_interval( $info{hostname}, "SERVICE",
	       $info{service_orig}, $config{NAGIOS_STATUS_FILE} );
	    if ($check_interval) {
	       &parse_perfdata( $object->{service}->{performance_data} );
	       &gen_graph_template();
	       &write_graph_template_to_file("$graph_template");
	    }
	 }
      } else {
                log_msg "Graph Template missing for service $info{service}\n" if $debug;
                return 0;
      }
   }

   # Check if a rewrite rules exist
   #
   if (
      -f "$config{CONF_DIR}/$config{TEMPLATES_DIR}/rewrite/$info{hostname}_$info{service}_rewrite"
     )
   {

      #
      #	print "Rewrite file detected for: ${hostname}_${service}\n";
      open REWRITE,
"<$config{CONF_DIR}/$config{TEMPLATES_DIR}/rewrite/$info{hostname}_$info{service}_rewrite"
	or log_msg "Can't open rewrite rules file for $info{hostname}\n"
	and exit 1;

      while (<REWRITE>) {
	 next if /^#/;    # Skip comments
	 next if /^$/;    # Skip empty lines
	 s/#.*//;         # Remove partial comments
	 chomp;

	 $rrd_file = $1 if /^rrd_file\s+(\S+)/;

	 #
	 # graph template
	 $graph_template = $1 if /^graph_template\s+(\S+)/;
      }
   }

   if ( defined $info{el} and $info{el} == 1 ) {
      if ( not defined( $info{ds_name} ) ) {
	 $graph_template =
"$config{CONF_DIR}/$config{TEMPLATES_DIR}/graph/$info{hostname}_$info{service}_el.t";
	 unless ( -e $graph_template ) {
	    $graph_template =
"$config{CONF_DIR}/$config{TEMPLATES_DIR}/graph/default_execution_latency_el.t";
	 }
      } else {
	$graph_template =
"$config{CONF_DIR}/$config{TEMPLATES_DIR}/graph/default_$info{ds_name}_el.t";
	
      }

      $rrd_file =~ s/\.rrd$/_el\.rrd/;
      if ( !-f $rrd_file ) {
	 $rrd_file =
	   "$config{RRA_DIR}/"
	   . (
	    defined $info{groupname}
	    ? $info{groupname}
	    : $info{hostname} )
	   . "/$info{hostname}_$info{service}_el.rrd";
      }
   }

   log_msg
"RRD file \"$rrd_file\" missing for host=\"$info{hostname}\" service=\"$info{service}\"\n"
     and exit 1
     if !-f $rrd_file;

   open G, "<$graph_template"
     or log_msg "($.) Can't open graph template \"$graph_template\"\n" and exit 1;

   my $demo_str = "";
   while (<G>) {
      next if /^#/;                                 # Skip comments
      next if /^$/;                                 # Skip empty lines
      next if /\$PREDICTION\$/ and $info{action};
      next if /\$NOW\$/ and $info{action};

      chomp;
      s/^\s+//;
      s/\s+$//;

      if (/\$HOSTNAME\$/) {
	 if ( defined $info{demo} ) {
	    $demo_str = random_string( $info{demo}, $config{DEMO_STR_LENGTH} );
	    s/\$HOSTNAME\$/$demo_str/g;
	 }
	 else {
	    s/\$HOSTNAME\$/$info{hostname}/g;
	 }
      }

      s/\$SERVICENAME\$/$info{service_orig}/g if /\$SERVICENAME\$/;
      s/\$RRD_FILENAME\$/$rrd_file/g          if /\$RRD_FILENAME\$/;
      s/\$DSNAME\$/$info{ds_name}/g           if /\$DSNAME\$/;

      #
      # Store graph height and width for later use
      #
      $graph_height = $1 if /(^-h|^--height)/ and m/(\d+)/;
      $graph_width  = $1 if /(^-w|^--width)/  and m/(\d+)/;

      #
      # looks like ethernet bandwidth calculation is enabled
      #
      $data_flow = 1 if /\$DATA_FLOW\$/;

      push @rrdtool_options, $_;

      s#$config{RRA_DIR}#PATH_HIDDEN# if $config{RRD_PATH_HIDDEN};

      # in case rrd file name is hard coded in template
      s#=.*/#=PATH_HIDDEN/# if /\.rrd/ and $config{RRD_PATH_HIDDEN};

      s/$info{hostname}/$demo_str/g if defined $info{demo};

      push @rrdtool_source, $_ if defined $info{source};
   }

   close G;

   my $now = localtime(time);
   $now =~ s/:/\\:/g;

   push @rrdtool_options, "COMMENT:\\n";
   push @rrdtool_options, "COMMENT:Create date\\: $now";
}
else {
   $config{SAVE_NAGIOS_ENVIRONMENT_VARS} = 0;
}

$col = 1;

if ( !defined $info{action} or ( $info{action} ne "image" ) ) {
   print "Content-Type: text/html\r\n\r\n";

   print "<head>\n\n";
   print "<title>\n\n";
   if ( defined $info{summary} ) {
      print "rrd2graph - summary \n\n";
   }
   else {
      print
	"rrd2graph - for host $info{hostname} and service $info{service} \n\n";
      $cache_dir =
	"$config{DOCUMENT_ROOT}/$config{CACHE_DIR}/"
	. (
	 defined $info{groupname}
	 ? $info{groupname}
	 : $info{hostname} );

      #
      # Creat hostname directory if necessary
      mkdir "$cache_dir" if !-d "$cache_dir";
   }
   print "</title>\n\n";
   print
"<meta http-equiv=refresh content='300'; url=\"/$config{CGIBIN}/rrd2graph.cgi?$ENV{QUERY_STRING}\">\n";
   print
'<meta http-equiv="Content-Type" content="text/html; charset=UTF-8"/>'; 
   print "<STYLE TYPE=\"text/css\">\n";
   print "<!--  \n";
   print
"BODY, TABLE, TR, TD {font-family: Verdana, Arial, Helvetica, sans-serif; font-size: 10px;}\n";
   print "BODY {background-color: #FFFFFF;}\n";
   print "A {text-decoration: none;}\n";
   print "A:active { text-decoration: none;}\n";
   print "A:hover {text-decoration: underline; color: #333333;}\n";
   print "A:visited {color: Blue;}\n";
   print ".textHeader {font-size: 12px; font-weight: bold;}\n";
   print
     ".textHeaderDark {font-size: 12px; font-weight: bold; color: #ffffff;}\n";
   print ".textArea {font-size: 12px;}\n";
   print "-->\n";
   print "</STYLE>\n";

   print <<H_JS

	    <script language="JavaScript">
	    <!--
	    function help(Click_Menu)
	    // Function that will swap the display/no display for
	    // all content within span tags
	    {
		    if (Click_Menu.style.display == "none") {
			    Click_Menu.style.display = "";
		    }
		    else {
			    Click_Menu.style.display = "none";
		    }
	    }
    -->
	    </script>
H_JS
     ;
   print "</head>\n\n";
   print "<html>\n\n";
   print '<body leftmargin="0" topmargin="0" marginwidth="0" marginheight="0">'
     . "\n\n";

   #
   # possibility to search by service/ds/host combination
   if ( defined $info{summary} ) {
      my $h_search = <<H_Search;
	    <span ID="h_search" style="display: none">
		    <table bgcolor="#FFFFCC">
		    <tr>
		    <td width=300 wrap>
		    <strong>
		    Service:
		    </strong> this field expects service name or substr of service name <br>
		    <strong>
		    DS Name:
		    </strong> this field expects exact DS (Data Source) Name, works only if Service is defined<br>
		    <strong>
		    Match Host:
		    </strong> this field expects host name or substr of host name<br>
		    </td>
		    </tr>
		    <td align='center'>
		    <a href=\"javascript:help(h_search)\">Close</a>
		    </td>
		    </table>
		    </span>
H_Search

      print
"<table align='center'><tr><td align='center' colspan=$config{THUMB_DISPLAY_COLUMNS}>";
      print "    <form  name=\"input\" action=\"/$config{CGIBIN}/rrd2graph.cgi\"
		    method=\"get\"> 
		    </strong><input type=\"hidden\" name=\"summary\" value=\"all\" 
		    ";

      print
"     <strong>Service: </strong> <input type=\"text\" name=\"service\" value=\""
	. ( defined $info{service} ? "$info{service}\">" : "\">" );
      print
"     <strong>DS name: </strong> <input type=\"text\" name=\"ds_name\" value=\""
	. ( defined $info{ds_name} ? "$info{ds_name}\">" : "\">" );

      print "     <input type=\"hidden\" name=\"el\"      value=\"1\">\n"
	if defined $info{el};
      print " <strong>Match host: </strong>
		    <input type=\"text\" name=\"match_host\" value=\""
	. ( defined $info{match_host} ? "$info{match_host}\">" : "\">" )
	if defined $info{summary};

      print " <SELECT NAME=\"display\"> ";

      foreach my $dg ( keys %graph ) {
	 next if $dg =~ /seconds/;
	 if ( $dg eq $config{THUMB_DISPLAY} ) {
	    print "<OPTION VALUE=\"$dg\" SELECTED> $dg  ";
	 }
	 else {
	    print "<OPTION VALUE=\"$dg\"> $dg";
	 }
      }
      print "
		</SELECT>
		<input type=\"submit\" value=\"Submit\">
		<a href=\"javascript:help(h_search)\">Help</a> $h_search</form></td></tr></table>";
   }

   print "<table align='center' width='1' cellpadding='0'>\n";
   print "<tr><td>\n" if $config{SAVE_NAGIOS_ENVIRONMENT_VARS};
}
$g = "";

if ( defined $info{summary} ) {

   use File::Find ();
   *dir   = *File::Find::dir;

   #
   # Summary related
   #
   File::Find::find( { wanted => \&wanted }, $config{RRA_DIR} );
   summary $info{summary};
}
else {

   if ( $config{SAVE_NAGIOS_ENVIRONMENT_VARS} ) {
      $nagios_env      = "";
      $nagios_env_file = $rrd_file;
      $nagios_env_file =~ s/(_el\.rrd|\.rrd)//;
      $nagios_env_file .= ".env";

      if ( -f $nagios_env_file ) {
	 load_nagios_env($nagios_env_file);
	 print "<table><tr><td>\n" if not defined $info{action};
      }
      else {
	 log_msg "Nagios environment vars file \"$nagios_env_file\" missing\n";
	 $config{SAVE_NAGIOS_ENVIRONMENT_VARS} = 0;
      }
   }

   use RRDs;

   #
   # Extract ds names from rrd_file
   $dshash = RRDs::info "$rrd_file";

   @dsnames = ();
   foreach my $key ( sort keys %$dshash ) {
      push @dsnames, $1 if $key =~ m/^ds\[(.*)\]\.type/;
   }

   foreach my $d ( "Daily", "Weekly", "Monthly", "Yearly" ) {
      $g = $d;

      $g = $info{display}
	if defined $info{display}
	   and defined $graph{ $info{display} };

      $total_graphs++;

      if ( defined $info{action} ) {
	 my ( $rrd_start, $step, $names, $data ) =
	   RRDs::fetch( "$rrd_file", "AVERAGE", "-s",
	    $graph{ $g . "_seconds" } * -1 );

	 if ( defined $info{graph_end} ) {
	    my $now = time;

	    $info{graph_end} = $now - $step
	      if $info{graph_end} > ( $now - $step );

	    # required for zoom out function
	    $info{graph_start}-- if $info{graph_start} == $info{graph_end};

	 }
	 $info{graph_start} = $rrd_start if !defined $info{graph_start};
	 $info{graph_end} = $rrd_start + ( ($#$data) * $step )
	   if !defined $info{graph_end};

	 zoom( $graph{ $g . "_seconds" }, $rrd_file );
	 last;
      }

      my $rrd_image_file = "$cache_dir/$info{hostname}_$info{service}_${g}.png";

      if ( $info{ds_name} ) {
	 $rrd_image_file =
	   "$cache_dir/$info{hostname}_$info{service}_$info{ds_name}_${g}.png"
	   if not $info{el};
	 $rrd_image_file =
"$cache_dir/$info{hostname}_$info{service}_$info{ds_name}_${g}_el.png"
	   if $info{el};
      }
      else {
	 $rrd_image_file =~ s/\.png$/_el\.png/ if $info{el};
      }

      foreach (@rrdtool_options) {
	 s/"//g;
	 s/\$DATA_FLOW\$/$graph{$g . "_seconds"}/g if $data_flow;
	 s/\$TREND\$/$graph{$g . "_seconds"}/g;
	 s/\$PREDICTION\$/--end=$config{"PREDICTION_$g"}/;
	 s/\$NOW\$/$now/;
      }

      #
      # Remove created images if demo variable is defined
      # so that, one doesn't have to remove images by hand
      unlink $rrd_image_file if defined $info{demo};
      gen_graph(
	 $info{hostname}, $info{service}, $rrd_image_file,
	 $graph{$g},      \@rrdtool_options
      );

      if ( $config{SAVE_NAGIOS_ENVIRONMENT_VARS} ) {
	 print
"<tr> <td align='left'><a href=\"/$config{CGIBIN}/rrd2graph.cgi?$ENV{QUERY_STRING}&extended_info=0\">Extended Information ON</a></td></tr>\n"
	   if $total_graphs == 1
	      and $config{NAGIOS_HOST_URL};
      }
      else {
	 print
"<tr> <td align='left'><a href=\"/$config{CGIBIN}/rrd2graph.cgi?$ENV{QUERY_STRING}&extended_info=1\">Extended Information OFF</a></td></tr>\n"
	   if $total_graphs == 1
	      and $config{NAGIOS_HOST_URL};
      }

      #
      # Diaplay source
      #
      if ( defined $info{source} ) {

	 #print "<tr> <td align='left'>\n";
	 print "<tr> <td valign=\"top\" bgcolor=\"#FFFFCC\">\n";
	 $rrd_image_file =~ s#$cache_dir#PATH_HIDDEN#
	   if $config{RRD_PATH_HIDDEN};

	 if ( defined $info{demo} ) {
	    my $demo_str =
	      random_string( $info{demo}, $config{DEMO_STR_LENGTH} );
	    $rrd_image_file =~ s/$info{hostname}/$demo_str/g;
	 }

	 print
	   "<PRE> $config{RRDTOOL} graph $rrd_image_file \\\n $graph{$g} \\\n";
	 foreach (@rrdtool_source) {
	    s/\$DATA_FLOW\$/$graph{$g . "_seconds"}/g if $data_flow;
	    s/\$TREND\$/$graph{$g . "_seconds"}/g;
	    s/\$PREDICTION\$/--end=$config{"PREDICTION_$g"}/;
	    s/\$NOW\$/$now/;

	    print;
	    print " \\\n";
	 }
	 print "</PRE> </td> </tr>\n";
      }

      #
      # Well any better solution to get the "Select DS" menu output ordered
      # is welcome
      print "<tr> <td align='center'>\n" if $#dsnames <= 0;
      print "<tr> <td align='right'>\n"  if $#dsnames > 0;

      if ( $info{el} ) {
	 print "<img src=\"/$config{CACHE_DIR}/"
	   . ( defined $info{groupname} ? $info{groupname} : $info{hostname} )
	   . "/$info{hostname}_$info{service}"
	   . ( $info{ds_name} ? "_$info{ds_name}_${g}_el.png\" " : "_${g}_el.png\" ")
	   . "alt=\"Plugin Execution and Latency graph image\"> <br> "
	   . "[<strong><a href=\"/$config{CGIBIN}/rrd2graph.cgi?hostname=$info{hostname}&"
	   . "service=$info{service_orig}&el=1&display=$g"
	   . ( $info{groupname} ? "&groupname=$info{groupname}" : "" )
	   . "\">$g</a></strong>] \n";
	 print
"[<a href=\"/$config{CGIBIN}/rrd2graph.cgi?hostname=$info{hostname}\&service=$info{service_orig}"
	   . ( $info{groupname} ? "&groupname=$info{groupname}" : "" )
	   . "\">Back to \"$info{service_orig}\"</a>] ";
	 print
" [<a href=\"/$config{NAGIOS_CGIBIN}/status.cgi?navbarsearch=1&host=$info{hostname}"
	   . ( $info{groupname} ? "&groupname=$info{groupname}" : "" )
	   . "\">Nagios</a>] "
	   if $config{NAGIOS_HOST_URL};
	 print
" [<a href=\"/$config{CGIBIN}/rrd2graph.cgi?hostname=$info{hostname}&service=$info{service}&"
	   . "el=1&display=$g"
	   . ( $info{groupname} ? "&groupname=$info{groupname}" : "" )
	   . ( $info{ds_name} ? "&ds_name=$info{ds_name}&source=1" : "&source=1")
	   . "\">Source</a>]"
	   . " [<a href=\"/$config{CGIBIN}/rrd2graph.cgi?action=zoom&hostname=$info{hostname}"
	   . "&service=$info{service_orig}&display=$g&graph_height=$graph_height&graph_width=$graph_width&el=1"
	   . ( $info{groupname} ? "&groupname=$info{groupname}" : "" )
	   . ( $info{ds_name} ? "&ds_name=$info{ds_name}\"" : "\"" )
	   . ">Zoom</a>]";
      }
      else {
	 print "<img src=\"/$config{CACHE_DIR}/"
	   . (  $info{groupname} ? $info{groupname} : $info{hostname} )
	   . "/$info{hostname}_$info{service}"
	   . ( $info{ds_name} ? "_$info{ds_name}_${g}.png\"" : "_${g}.png\"" )
	   . " alt=\"$g graph image\"> <br> <strong>[<strong><a href=\"/$config{CGIBIN}/rrd2graph.cgi?"
	   . "hostname=$info{hostname}\&service=$info{service_orig}&display=$g"
	   . ( $info{groupname} ? "&groupname=$info{groupname}" : "" )
	   . "\">$g</a></strong>] \n";
	 if (
	    "$config{DOCUMENT_ROOT}/$config{CACHE_DIR}/"
	    . ( $info{groupname} ? $info{groupname} : $info{hostname})
	    . "/$info{hostname}_$info{service}_${g}_el.png")
	 {
	    print
"[<a href=\"/$config{CGIBIN}/rrd2graph.cgi?hostname=$info{hostname}\&service=$info{service_orig}"
	      . ( $info{groupname} ? "&groupname=$info{groupname}" : "" )
	      . "&el=1\">Plugin E/L Time</a>] ";
	    print
"[<a href=\"/$config{NAGIOS_CGIBIN}/status.cgi?navbarsearch=1&host=$info{hostname}\">Nagios</a>] "
	      if $config{NAGIOS_HOST_URL};
	    print
" [<a href=\"/$config{CGIBIN}/rrd2graph.cgi?hostname=$info{hostname}&service=$info{service_orig}"
	      . ( $info{groupname} ? "&groupname=$info{groupname}" : "" )
	      . "&display=$g"
	      . ( $info{ds_name} ? "&ds_name=$info{ds_name}&source=1" : "&source=1")
	      . "\">Source</a>]"
	      . " [<a href=\"/$config{CGIBIN}/rrd2graph.cgi?action=zoom&hostname=$info{hostname}"
	      . "&service=$info{service_orig}&display=$g&graph_height=$graph_height&graph_width=$graph_width&el=0"
	      . ( $info{groupname} ? "&groupname=$info{groupname}" : "" )
	      . ( $info{ds_name} ? "&ds_name=$info{ds_name}\"" : "\"" )
	      . ">Zoom</a>]";
	 }
      }
      print
" [Graphs <a href=\"/$config{CGIBIN}/rrd2graph.cgi?summary=$info{hostname}\">Host</a>"
	. "|<a href=\"/$config{CGIBIN}/rrd2graph.cgi?summary=all\">All</a>]";

#
# If more than one DS in rrd_file then display "Select DS" menu
      if ( $#dsnames > 0 ) {
	 print
	   " [<a href=\"javascript:help(ds_menu_$g)\">Select DS</a>]<br><br>";
	 $ds_menu = "<span ID=\"ds_menu_$g\" style=\"display: none\">"
	   . "<table bgcolor=\"#FFFFCC\">";

	 $ds_menu .=
"<tr><td width=25 align=left><strong>$info{service_orig}</strong></td><td>";
	 $ds_menu .=
"<a href=\"/$config{CGIBIN}/rrd2graph.cgi?hostname=$info{hostname}&service=$info{service_orig}";
	 $ds_menu .= ( $info{groupname} ? "&groupname=$info{groupname}" : "" );
	 $ds_menu .= ( $info{el} ? "&el=1" : "" );
	 $ds_menu .= "\">$info{hostname}</a>|";
	 my $strip_svc = $info{service_orig};
	 $strip_svc =~ s/^_//;
	 $ds_menu .=
"<a href=\"/$config{CGIBIN}/rrd2graph.cgi?summary=all&service=$strip_svc";
	 $ds_menu .= ( $info{groupname} ? "&groupname=$info{groupname}" : "" );
	 $ds_menu .= ( $info{el} ? "&el=1" : "" );
	 $ds_menu .= "\">All</a> </td> </tr>";

	 foreach my $ds (@dsnames) {
	    $ds_menu .= "<tr><td width=25 align=left>$ds</td><td>";
	    $ds_menu .=
"<a href=\"/$config{CGIBIN}/rrd2graph.cgi?hostname=$info{hostname}&service=$info{service_orig}";
	    $ds_menu .= ( $info{groupname} ? "&groupname=$info{groupname}" : "" );
	    $ds_menu .= ( $info{el} ? "&el=1" : "" );
	    $ds_menu .= "&display=$g&ds_name=$ds\">$g</a>|";
	    $ds_menu .=
"<a href=\"/$config{CGIBIN}/rrd2graph.cgi?hostname=$info{hostname}&service=$info{service_orig}";
	    $ds_menu .= ( $info{groupname} ? "&groupname=$info{groupname}" : "" );
	    $ds_menu .= ( $info{el} ? "&el=1" : "" );
	    $ds_menu .= "&ds_name=$ds\">DWMY</a>|";
	    $ds_menu .=
"<a href=\"/$config{CGIBIN}/rrd2graph.cgi?summary=all&service=$info{service_orig}";
	    $ds_menu .= ( $info{groupname} ? "&groupname=$info{groupname}" : "" );
	    $ds_menu .= ( $info{el} ? "&el=1" : "" );
	    $ds_menu .= "&ds_name=$ds\">All</a> </td> </tr>";
	 }
	 $ds_menu .= "</table> </span>";
	 print $ds_menu;
      }
      else {
	 print "<br><br>\n";
      }
      print "</td> </tr>\n";
      last if defined $info{source};
      last if $info{display} and defined $graph{ $info{display} };
   }
}

if ( !defined $info{graph_end} ) {
   print "</table> \n";
   if ( $config{SAVE_NAGIOS_ENVIRONMENT_VARS} ) {
      print "</td><td valign='top'> $nagios_env </td></tr></table>\n"
	and $nagios_env_count++
	if not $nagios_env_count;
   }
   print "$footer" . $total_graphs;
   print "</body>\n\n";
   print "</html>\n\n";
}

sub log_msg ($) {
   my $str = shift;

   if ($debug) {
      open L, ">>$config{LOGFILE}"
	or die "Can't open logfile \"$config{LOGFILE}\"";
      print L scalar localtime() . ":rrd2graph: $str";
      close L;
   }
   else {
      print "Content-Type: text/html\r\n\r\n";
      print "<head>\n\n";
      print "<title>\n\n";
      print "n2rrd (rrd2graph) \n\n";
      print "</title>\n\n";

      print "</head>\n\n";
      print "<html>\n\n";
      print "<body>\n\n";
      print "<HR> <H2> rrd2graph:ERROR: </H2><BR>\n";
      print "<P align=left>\n";
      print "	<FONT face=\"Comic Sans MS\" size=4> \n";
      print "$str";
      print "<BR><BR>\n</FONT>\n</P>\n ";
      print "$footer" . $total_graphs;
      print "</body>\n\n";
      print "</html>\n\n";
   }
}

sub summary ($) {
   my $op = shift;

   if ( $op eq "all" ) {

      foreach my $h ( sort keys %{$rrd_files} ) {

	 my $found_env = 0;
	 my $tmp_s     = "";
	 $tmp_rrd_files = {};
	 if ( ( defined $info{extmacro} ) and ( defined $info{extvalue} ) ) {
	    foreach my $s ( sort keys %{ $rrd_files->{$h} } ) {
	       $tmp_s = $s;
	       $s =~ s/rrd/env/;
	       my $env_file = "$config{RRA_DIR}/$rrd_dirnames->{${h}}_$s";
	       open( ENV, "<$env_file" ) or next;
	       while (<ENV>) {
		  chomp;
		  my ( $k, $v ) = split /=/, $_, 2;
		  if (   ( $k eq $info{extmacro} )
		     and ( $v eq $info{extvalue} ) )
		  {
		     $found_env = 1;
		     $tmp_rrd_files->{$h}->{$tmp_s}++;
		     last;
		  }
	       }
	       close ENV;

	       #last if $found_env;
	    }
	    if ($found_env) {
	       display_host $h, \%{$tmp_rrd_files};
	    }
	    next;
	 }
	 display_host $h, \%{$rrd_files};
      }
   }
   else {
      if ( defined $rrd_files->{$op} ) {
	 display_host $op, \%{$rrd_files};
	 if ( $col < 4 ) {
	    print "\t<td> </td>\n<td> </td>\n</tr>\n" if $col == 2;
	    print "\t<td> </td>\n</tr>\n"             if $col == 3;
	 }
      }
      else {
	 print "<HR> <H2> rrd2graph:ERROR: </H2><BR>\n";
	 print "<P align=left>\n";
	 print " <FONT face=\"Comic Sans MS\" size=4> \n";
	 print "This host \"$op\" does not exist";
	 print "<BR><BR>\n</FONT>\n</P>\n\n";
      }
   }
}

# Call by subroutine summary 
sub display_host ($$) {
   my $host         = shift;
   my $ref          = shift;
   my $use_rrd_file = "";
   my $tt           = "";
   my $graph_t      = "";

   foreach my $s ( sort keys %{ $ref->{$host} } ) {
      my $t = $s;
      $t =~ s/\.rrd//;
      $tt = $t;

      # Search for the original service, which is used later in &get_check_interval

      foreach my $key ( keys(%service_map)) {
        $tt = $key if ($service_map{$key} eq $t) ;
      }

      if ( $info{el} ) {
	 $s = "$t" . "_el" . ".rrd";
	 $t .= "_el";
      }
      else {
	 $t .= "_" . $info{ds_name} if $info{ds_name};
      }

      print "<tr>\n" if $col == 1;

      if ( -f "$config{CONF_DIR}/$config{TEMPLATES_DIR}/graph/${host}_${t}.t" )
      {
	 $graph_t =
	   "$config{CONF_DIR}/$config{TEMPLATES_DIR}/graph/${host}_${t}.t";
      }
      elsif ( -f "$config{CONF_DIR}/$config{TEMPLATES_DIR}/graph/${t}.t" ) {
	 $graph_t = "$config{CONF_DIR}/$config{TEMPLATES_DIR}/graph/${t}.t";
      }
      elsif ( $config{DYN_GRAPH_CREATE} ) {
	 $graph_t =
"$config{CONF_DIR}/$config{TEMPLATES_DIR}/graph/dyn/${host}/${host}_${t}.t";
         my $tmp_service_description = undef;
	 $tmp_service_description = $1 if ${t} =~ m/^([^_]+)/;

         if (! defined  $tmp_service_description ) {
             my @parse_service_str = split /_/, $info{service};
                if ( $config{TEMPLATE_NAME_AT_BEGINNING} ) {
	 		# The first word after terminator character "_"
	 		$tmp_service_description = $parse_service_str[0];
      		}
      		else {
	 		# The last word after terminator character "_"
	 		$tmp_service_description = $parse_service_str[$#parse_service_str];
      		}
         }

	 if ( !-f $graph_t ) {    # create template if doesn't exist
	    mkdir "$config{CONF_DIR}/$config{TEMPLATES_DIR}/graph/dyn"
	      if !-d "$config{CONF_DIR}/$config{TEMPLATES_DIR}/graph/dyn";

	    mkdir "$config{CONF_DIR}/$config{TEMPLATES_DIR}/graph/dyn/${host}"
	      if
	       !-d "$config{CONF_DIR}/$config{TEMPLATES_DIR}/graph/dyn/${host}";

	    my $check_interval =
	      &get_check_interval( ${host}, "SERVICE", $tmp_service_description,
	       $config{NAGIOS_STATUS_FILE} );
	    if ($check_interval) {
	       &parse_perfdata( $object->{service}->{performance_data} );
	       if ( $info{ds_name} ) {
		  &gen_graph_template( $info{ds_name} );
	       }
	       else {
		  &gen_graph_template();
	       }
	       &write_graph_template_to_file("$graph_t");
	    }
	 }
      }
      else {
                log_msg "Graph Template missing for service $t\n" if $debug;
                return 0;
      }


      if ($info{el}) {
      	if ($info{ds_name}) {
      	  $graph_t = "$config{CONF_DIR}/$config{TEMPLATES_DIR}/graph/default_$info{ds_name}_el.t";
	} else {
      	  $graph_t = "$config{CONF_DIR}/$config{TEMPLATES_DIR}/graph/default_execution_latency_el.t";
	}
      }

      if ( -e "$graph_t" ) {
	 $cache_dir = "$config{DOCUMENT_ROOT}/$config{CACHE_DIR}/$rrd_dirnames->{${host}}";

	 mkdir "$cache_dir" if !-d "$cache_dir";

	 create_thumb $host,
	   ( $info{ds_name} ? "$tt->$info{ds_name}" : $tt ), "$graph_t",
	   "$config{RRA_DIR}/$rrd_dirnames->{${host}}/${host}_$s",
	   "$config{DOCUMENT_ROOT}/$config{CACHE_DIR}/$rrd_dirnames->{${host}}/${host}_${t}_tb.png";

	 if (
	    -e "$cache_dir/${host}_${t}_tb.png"
	   )
	 {
	    print
"\t<td> <a href=\"/$config{CGIBIN}/rrd2graph.cgi?hostname=$host&service="
	      . ( $info{ds_name} ? "$tt&ds_name=$info{ds_name}" : $tt )
	      . "&groupname=$rrd_dirnames->{${host}}"
	      . ( $info{el}      ? "&el=1"                      : "" )
	      . ( $info{display} ? "&display=$info{display}\">" : "\">" )
	      . "<img src=\"/$config{CACHE_DIR}/$rrd_dirnames->{${host}}/${host}_${t}_tb.png\"> </a> </td>\n";
	    $col++;
	 }

	 print "</tr>\n" and $col = 1
	   if $col == ( $config{THUMB_DISPLAY_COLUMNS} + 1 );
	 $total_graphs++;
      }
   }
}

sub create_thumb ($$$$$) {
   my ( $host, $service, $template, $rrd_file, $thumb_image_filename ) = @_;

   my $rrd_start = $graph{ $config{THUMB_DISPLAY} };
   $rrd_start = $graph{ $info{display} }
     if defined $info{display} and defined $graph{ $info{display} };

   use RRDs;

   @rrdtool_options = ();

   push @rrdtool_options, "--no-legend";

   open G_TEMP, "<$template"
     or die "Can't open graph template \"$template\"" and exit 1;

   while (<G_TEMP>) {
      next if /^#/;                     # Skip comments
      next if /^$/;                     # Skip empty lines
      next if /-z/ and $info{el};
      next if /--lazy/ and $info{el};
      next if /\$PREDICTION\$/;
      next if /\$NOW\$/;
      chomp;

      s/^\s+//;
      s/\s+$//;
      s/"//g;

      my $demo_str = "";
      if (/\$HOSTNAME\$/) {
	 if ( defined $info{demo} ) {
	    $demo_str = random_string( $info{demo}, $config{DEMO_STR_LENGTH} );
	    s/\$HOSTNAME\$/$demo_str/g;
	 }
	 else {
	    s/\$HOSTNAME\$/$info{hostname}/g if defined $info{hostname};
	 }
      }

      s/\$RRD_FILENAME\$/$rrd_file/g if /\$RRD_FILENAME\$/;
      s/\$DSNAME\$/$info{ds_name}/g  if /\$DSNAME\$/;

      #
      # remove width and height, and provide new one one
      $_ = "--width=$config{THUMB_WIDTH}" if /-w\s+/;
      $_ = "--width=$config{THUMB_WIDTH}" if /--width=/;

      $_ = "--height=$config{THUMB_HEIGHT}" if /-h\s+/;
      $_ = "--height=$config{THUMB_HEIGHT}" if /--height=/;

      $host = $demo_str if $info{demo};

      $_ = "--title=$host - $service" if /-t\s+/;
      $_ = "--title=$host - $service" if /--title=/;

      $_ =~ s/\$DATA_FLOW\$/86400/g if /\$DATA_FLOW\$/;

      if ( defined $info{display} ) {
	 $_ =~ s/\$TREND\$/$graph{ $info{display} . "_seconds" }/g
	   if /\$TREND\$/;
      }
      else {
	 $_ =~ s/\$TREND\$/$graph{ $config{THUMB_DISPLAY} . "_seconds" }/g
	   if /\$TREND\$/;
      }
      push @rrdtool_options, $_;
   }

   #log_msg "@rrdtool_options" if $debug;

   close G_TEMP;

   #
   # remove image if default display value changes temporarly
   if ( ( defined $info{display} and defined $config{THUMB_DISPLAY} ) ) {
      unlink $thumb_image_filename
	if $info{display} ne $config{THUMB_DISPLAY};
   }

   #
   # makes sense to remove the graph in demo mode
   unlink $thumb_image_filename if defined $info{demo};
   gen_graph( $host, $service, $thumb_image_filename, $rrd_start,
      \@rrdtool_options );

   return 0;
}

sub wanted {
   my ( $dev, $ino, $mode, $nlink, $uid, $gid, $host, $service );

   ( ( $dev, $ino, $mode, $nlink, $uid, $gid ) = lstat($_) );

   #
   # Store hostname and service name
   #
   if ( -f _ ) {
      if ( $_ =~ /_el\.rrd$/s ) {
	 return;
      }
      elsif ( $_ =~ /^(.*)_([A-Za-z0-9\-\s]+\.rrd)$/s ) {
	 $host    = $1;
	 $service = $2;
      }

      #( $host, $service ) = split '_', $_, 2;
      return if !$service;
      return if $info{service} and $service !~ /$info{service}/ and $service !~ $service_map{$info{service}} ;
      return if defined $info{match_host} and $host !~ /$info{match_host}/;
      $rrd_files->{$host}->{$service}++ if -f $_;
      $dir =~ s#.*/##;
      $rrd_dirnames->{$host} = $dir;
   }
}

sub zoom ($$) {
   my ( $start, $rrd_filename ) = @_;

   #
   # for bandwidth calculation
   #
   if ($data_flow) {
      my ( $rrd_start, $step, $names, $data ) =
	RRDs::fetch( "$rrd_filename", "AVERAGE", "-s", $info{graph_start}, "-e",
	 $info{graph_end} );

      my $end = $rrd_start + ( ($#$data) * $step );
      $start = $end - $rrd_start;
   }

   $info{el} = 0 if $info{el} != 1;

   my $help_text = <<H_TEXT;
		<span ID="zoom_help" style="display: none">
		<table bgcolor="#FFFFCC">
		<tr>
		<td width=300 wrap>
		<strong>
		Zoom in: 
		</strong> with "left mouse key" pressed, mark graph area and release key. <br>
		<strong>
		Zoom out:
		</strong> click graph area with "right mouse key"
		</td>
		</tr>
		</table>
		</span>
H_TEXT

   if ( $info{action} eq "zoom" ) {
      print <<DIV;
			<div id='zoomBox' style='position:absolute; overflow:none; left:0px; top:0px; width:0px; height:0px; visibility:visible; background:red; filter:alpha(opacity=50); -moz-opacity:0.5; -khtml-opacity:0.5; opacity:0.5'></div>
			<div id='zoomSensitiveZone' style='position:absolute; overflow:none; left:0px; top:0px; width:0px; height:0px; visibility:visible; cursor:crosshair; background:blue; filter:alpha(opacity=0); -moz-opacity:0; -khtml-opacity:0;opacity:0' oncontextmenu='return false' ></div>
			<STYLE MEDIA="print">
			/*Turn off the zoomBox*/
			div#zoomBox, div#zoomSensitiveZone {display: none}
		/*This keeps IE from cutting things off*/
#why {position: static; width: auto}
		</STYLE>

DIV

      print "<tr align='center'><td><strong>Zoom graph => Start: "
	. scalar localtime( $info{graph_start} )
	. ", End: "
	. scalar localtime( $info{graph_end} )
	. " , graph Height=$graph_height Width=$graph_width</strong><br></td></tr>\n";

      print "<tr><td align='center'>\n";
      my $zoom_start = 2 * $info{graph_start} - $info{graph_end};
      my $zoom_end   = 2 * $info{graph_end} - $info{graph_start};
      print
"<a href='/$config{CGIBIN}/rrd2graph.cgi?action=zoom&graph_start=$zoom_start&graph_end=$info{graph_start}&graph_height=$graph_height&graph_width=$graph_width&hostname=$info{hostname}&service=$info{service_orig}&title_font_size=10&el=$info{el}"
	. ( $info{groupname} ? "&groupname=$info{groupname}" : "" )
	. ( $info{ds_name} ? "&ds_name=$info{ds_name}'>" : "'>" )
	. " << Previous</a> | ";
      print
"<a href='/$config{CGIBIN}/rrd2graph.cgi?action=zoom&graph_start=$info{graph_end}&graph_end=$zoom_end&graph_height=$graph_height&graph_width=$graph_width&hostname=$info{hostname}&service=$info{service_orig}&title_font_size=10&el=$info{el}"
	. ( $info{groupname} ? "&groupname=$info{groupname}" : "" )
	. ( $info{ds_name} ? "&ds_name=$info{ds_name}'>" : "'>" )
	. "Next >> </a>";
      print "</td></tr>\n";

      print "<tr><td>\n";
      if ( $info{ds_name} ) {
	 print <<DSIMG;
			<img id='zoomGraphImage' src='/$config{CGIBIN}/rrd2graph.cgi?action=image&graph_start=$info{graph_start}&graph_end=$info{graph_end}&graph_height=$graph_height&graph_width=$graph_width&hostname=$info{hostname}&service=$info{service_orig}&title_font_size=10&el=$info{el}&ds_name=$info{ds_name}&groupname=$info{groupname}' border='0' alt='$info{hostname} - $info{service}'> 
DSIMG
      }
      else {
	 print <<IMG;
			<img id='zoomGraphImage' src='/$config{CGIBIN}/rrd2graph.cgi?action=image&graph_start=$info{graph_start}&graph_end=$info{graph_end}&graph_height=$graph_height&graph_width=$graph_width&hostname=$info{hostname}&service=$info{service_orig}&title_font_size=10&el=$info{el}&groupname=$info{groupname}' border='0' alt='$info{hostname} - $info{service}'> 
IMG
      }
      print "</td></tr><br>\n";
      print "<tr align='center'><td>";
      print
"[<a href=\"/$config{CGIBIN}/rrd2graph.cgi?hostname=$info{hostname}&service=$info{service_orig}&el=$info{el}"
	. ( $info{groupname} ? "&groupname=$info{groupname}" : "" )
	. ( $info{ds_name} ? "&ds_name=$info{ds_name}\">" : "\">" )
	. "Back</a>]";
      print " [<a href=\"javascript:help(zoom_help)\">Help</a>]";
      print $help_text;
      if ( $config{OPTIONAL_IMAGE_FORMATS} ) {

      my $download_menu = <<DOWNLOAD_TEXT;
		<span ID="download_menu" style="display: none">
		<table bgcolor="#FFFFCC" width="10" align="right">
DOWNLOAD_TEXT

      # supported by rrdtool
      my @IMAGE_FORMATS = ( "PNG", "PDF", "EPS", "SVG" );
      foreach my $format (@IMAGE_FORMATS) {
           $download_menu .= " <tr> <td> [<a href=\"/$config{OPTIONAL_CGIBIN2}/rrd2graph.cgi?action=image&graph_start=$info{graph_start}&graph_end=$info{graph_end}&graph_height=$graph_height&graph_width=$graph_width&hostname=$info{hostname}&service=$info{service}&title_font_size=10&el=$info{el}&groupname=$info{groupname}&image_format=$format"
	. ( $info{ds_name} ? "&ds_name=$info{ds_name}\">" : "\">" )
	. "$format</a>] </td> </tr>\n";
      }
      $download_menu .= "</table> </span>";
      print " [<a href=\"javascript:help(download_menu)\">Download</a>]";
      print $download_menu;
      }
      print "</td><tr></table> \n";

      #
      # load zoom java script
      open JS, "< $config{CONF_DIR}/$config{ZOOM_JS}"
	or log_msg
	"Can't open zoom script \"$config{CONF_DIR}/$config{ZOOM_JS}\"\n"
	and exit 1;

      while (<JS>) {
	 s/\@CGIBIN\@/$config{CGIBIN}/;
	 push @zoom_javascript, $_;
      }
      close JS;
      print @zoom_javascript;
      print "$footer" . $total_graphs;
      print " <br></body>\n\n";
      print "</html>\n\n";
   }
   elsif ( $info{action} eq "image" ) {
      use File::Basename;
      
      my $content_filename = basename($rrd_filename, ".rrd");
         $content_filename .=  "_" . $info{ds_name} if  $info{ds_name};

      foreach (@rrdtool_options) {
	 s/"//g;
	 s/\$DATA_FLOW\$/$start/g if $data_flow;
         s/--imgformat=PNG/--imgformat=$info{image_format}/ and $imgformat_defined++ if defined $info{image_format};
         s/^-a\s+.*/--imgformat=$info{image_format}/ and $imgformat_defined++ if defined $info{image_format};

	 if ( defined $info{display} ) {
	    $_ =~ s/\$TREND\$/$graph{ $info{display} . "_seconds" }/g;
	 }
	 else {
	    $_ =~ s/\$TREND\$/$graph{ $config{THUMB_DISPLAY} . "_seconds" }/g;
	 }
      }
      if (defined $info{image_format}){
	unshift(@rrdtool_options, "--imgformat=$info{image_format}") if $imgformat_defined == 0;
      }

      my ( $fh, $filename );
      $filename = '-';

      # default mime info
      my $mime_type="image/png";
      my $mime_file_ext="png";

      if (defined $info{image_format}) {
       if ($info{image_format} eq 'SVG') {
	$mime_type="image/svg+xml";
	$mime_file_ext="svg";
       }
       elsif ($info{image_format} eq 'PDF') {
	$mime_type="application/pdf";
	$mime_file_ext="pdf";
       }
       elsif ($info{image_format} eq 'EPS') {
	$mime_type="application/postscript";
	$mime_file_ext="eps";
       }
      }

      if ( $ENV{MOD_PERL} ) {    # This variable is set by mod_perl
        use CGI qw/:standard :imagemap/;
          print header(-expires=>'now',
                       -type => $mime_type,
                       -content_disposition => "inline:filename=${content_filename}.${mime_file_ext}");

	 use File::Temp qw/ tempfile /;
	 ( $fh, $filename ) = tempfile(
	    DIR    => $config{TMPDIR},
	    SUFFIX => '.image',
	 );
      } else {
      	print "Cache-Control: max-age=0\n";
      	print "Content-Type: $mime_type\n";
      	print "Content-Disposition: attachement; filename=${content_filename}.${mime_file_ext}\n\n";
      }

      if ( $RRDs::VERSION >= 1.2013 ) {
	 push @rrdtool_options,
	     "--watermark="
	   . scalar localtime( $info{graph_start} ) . " to "
	   . scalar localtime( $info{graph_end} );
      }

      my ( $a, $xsize, $ysize ) =
	RRDs::graph( $filename, "--start=$info{graph_start}",
	 "--end=$info{graph_end}", @rrdtool_options );

      #
      # This work around is needed as RRDs::graph writes to STDOUT
      # under mod_perl the data is not flushed properly
      #
      if ( $ENV{MOD_PERL} ) {
	 binmode $fh;
	 my $buf;
	 while ( sysread $fh, $buf, 8192 ) {
	    print $buf;
	 }
	 close $fh;
	 unlink $filename;
      }

      $RRD_ERROR = RRDs::error;
      log_msg "ERROR \"hostname = $info{hostname} -> service = $info{service}\"\
			graph failed : $RRD_ERROR\n" if $RRD_ERROR;
   }
   else {
      log_msg "zoom: unknown type \"$info{action}\"\n";
   }
}

sub load_config ($) {
   my $config_file = shift;
   my $line        = 0;

   open( CONF, "<$config_file" )
     or die "Can't open configuration file -> \"$config_file\":$!\n";

   #
   # Parse configuration file
   while (<CONF>) {
      $line++;    # note confirguration file line number
      chomp;      # Remove newline character
      s/\s//g;     # remove spaces
      next if /^\s*#/;    # Skip comments
      next if /^\s*$/;    # Skip empty lines
      s/#.*//;         # Remove partial comments

      if (/^(.*)=(.*)$/) {
	 if ( defined $config{$1} ) {
	    if ( $1 eq "THUMB_DISPLAY_COLUMNS" ) {
	       if ( ( $2 == 2 ) or ( $2 == 3 ) ) {
		  $config{$1} = $2;
	       }
	       else {
		  log_msg
"Wrong value supplied to \"THUMB_DISPLAY_COLUMNS\" must be 2 or 3\n";
	       }
	    }
	    else {
	       $config{$1} = $2;
	    }
	 }
	 else {    # A warning should be ok
	    log_msg
"WARNING: Unknown global variable declaration in line $line:\"$1 = $2\"\n" if $debug;
	 }
      }
   }
   close CONF;
}

sub gen_graph ($$$$$) {
   my (
      $host_name,      $service_name, $image_filename,
      $rrd_start_date, $rrd_options
   ) = @_;

   my ( $a, $xsize, $ysize ) =
     RRDs::graph( $image_filename, $rrd_start_date, @{$rrd_options} );

   $RRD_ERROR = RRDs::error;
   log_msg
"ERROR \"hostname = $host_name -> service = $service_name\" graph failed : $RRD_ERROR\n"
     if $RRD_ERROR;
   return 0;
}

sub load_nagios_env ($) {
   my $env_file = shift;

   my @env_choice = ();
   my %tmp        = ();

   #
   # read environment choice from template
   #
   my $env_template = "$config{CONF_DIR}/$config{TEMPLATES_DIR}/environment.t";
   open( E_TEMP, "<$env_template" )
     or die "Can't open Environment template file \"$env_template\"";
   while (<E_TEMP>) {
      chomp;     # Remove newline character
      s/ //g;    # remove spaces
      next if /^#/;               # Skip comments
      next if /^$/;               # Skip empty lines
      next if $_ !~ /^NAGIOS/;    # ignore junk
      s/#.*//;                    # Remove partial comments

      push @env_choice, $_;
   }
   log_msg "Environment template \"$env_template\" has no valid value\n"
     and exit 1
     if $#env_choice < 0;

   open( E, "<$env_file" )
     or die "Can't open Environment file -> \"$env_file\":$!\n";

   #
   # load environment variables
   while (<E>) {
      chomp;    # Remove newline character

      my ( $k, $v ) = split /=/, $_, 2;

      $v = localtime($v)
	if $k =~
/(LASTHOSTDOWN|LASTSERVICEOK|LASTSERVICECHECK|LASTHOSTCHECK|TIMET|LASTSERVICESTATECHANGE|LASTHOSTSTATECHANGE|LASTHOSTUP|EVENTSTARTTIME|EVENTSTARTTIME|LASTSERVICECRITICAL|PROCESSSTARTTIME)/;
      $tmp{$k} = $v;
   }
   close E;

   $nagios_env = "<table align='left' cellpadding='0'>\n";
   $nagios_env .=
"<br><tr><td bgcolor=\"gray\">MACRO NAME</td><td bgcolor=\"gray\">Value</td><tr>\n";
   foreach my $i (@env_choice) {
      my $result_str_color = "lightgrey";
      $result_str_color = 'red'
	if defined $tmp{$i} and $tmp{$i} =~ m/^(DOWN|CRITICAL)/i;
      $result_str_color = 'yellow'
	if defined $tmp{$i} and $tmp{$i} =~ m/^WARNING/i;
      $result_str_color = 'lightgreen'
	if defined $tmp{$i} and $tmp{$i} =~ m/^(UP|OK)/i;
      $nagios_env .=
"<tr><td bgcolor=#ffe0dd>$i</td><td nowrap bgcolor=$result_str_color>$tmp{$i}</td></tr>\n"
	if defined $tmp{$i};
   }
   $nagios_env =~ s/NAGIOS_//g;
   $nagios_env .= "</table>\n";
}

sub list_default_values() {
   load_config($conf_file) if $conf_file;

   exit 0 if $ENV{'REQUEST_METHOD'};
   printf "#\n# Generated by rrd2graph: $version\n";
   printf "#%24s = %s\n", "Key  ", "  Value";
   printf "%25s = %s\n", "#" x 25, "#" x 30;
   foreach my $k ( sort keys %config ) {
      printf "%25s = %s\n", $k, $config{$k};
   }
   exit 0;
}

#
# Generate random string, which would replace HOSTNAME
#
sub random_string($$) {
   my $prepend = shift || "demo";
   my $length  = shift || $config{DEMO_STR_LENGTH};
   my @num     = ( 0 .. 9 );
   my $randstr = join '', map $num[ rand @num ], 0 .. $length;

   return "${prepend}${randstr}";
}

sub gen_graph_template($) {
   my $dsname = shift;

   &load_rgb_colors("$config{CONF_DIR}/$config{DYN_RGB_COLORS_MAPS}")
     if -f "$config{CONF_DIR}/$config{DYN_RGB_COLORS_MAPS}";
   my $max_colors = $#colors;
   my $i          = 0;

   # reset global variable rrd_string
   @rrdtool_options = ();

   # generate DS info
   push @rrdtool_options, "#Dynamically Generated graph template";
   push @rrdtool_options, "--imgformat=PNG";
   push @rrdtool_options, "--lazy";
   push @rrdtool_options, "--base=1024";
   push @rrdtool_options, "--height=200";
   push @rrdtool_options, "--width=500";
   push @rrdtool_options, "--alt-autoscale-max";
   push @rrdtool_options, "--lower-limit=0";
   push @rrdtool_options, "--slope-mode";

   if ($dsname) {
      push @rrdtool_options,
	"--title=\"\$HOSTNAME\$ (\$SERVICENAME\$ -> $dsname)- Usage\"";
      foreach my $l ( keys %{$perfdata} ) {
	 next
	   unless $perfdata->{$l}->{clean} =~ /$dsname/
	      or $l =~ /$dsname/;
	 my $ds = $perfdata->{$l}->{clean};

	 my $t_uom = "";
	 if ( $perfdata->{$l}->{uom} and $perfdata->{$l}->{uom} !~ /c/i ) {
	    $t_uom = $perfdata->{$l}->{uom};
	    push @rrdtool_options, "--vertical-label=\"usage in $t_uom\"";
	    $t_uom =~ s/%/%%/g;    # for values marked as percent
	    $t_uom = "%s$t_uom";
	 }
	 push @rrdtool_options, "DEF:$ds=\$RRD_FILENAME\$:$ds:AVERAGE";
	 push @rrdtool_options, "COMMENT:\"\\n\"";
	 push @rrdtool_options,
	   "COMMENT:\"  Current      Average     Maximum\\n\"";
	 push @rrdtool_options, "GPRINT:$ds:LAST:\"%8.2lf $t_uom\"";
	 push @rrdtool_options, "GPRINT:$ds:AVERAGE:\"%8.2lf $t_uom\"";
	 push @rrdtool_options, "GPRINT:$ds:MAX:\"%8.2lf $t_uom\"";
	 push @rrdtool_options, "#Original label = " . "$l";
	 push @rrdtool_options,
	   "AREA:${ds}#${config{DYN_GRAPH_USED_VALUE_COLOR}}:$l\:";
	 push @rrdtool_options, "LINE:$perfdata->{$l}->{max}#000000:Max marker:"
	   if $perfdata->{$l}->{max};
	 last;
      }
   }
   else {
      push @rrdtool_options,
	"--title=\"\$HOSTNAME\$ (\$SERVICENAME\$)- Usage\"";
      my $check = 0;
      foreach my $l ( keys %{$perfdata} ) {
	 my $ds = $perfdata->{$l}->{clean};
	 push @rrdtool_options,
	   "COMMENT:\"  Current      Average     Maximum\\n\"" and $check++
	   if $check == 0;
	 push @rrdtool_options, "#Original label = " . "$l";
	 push @rrdtool_options, "DEF:$ds=\$RRD_FILENAME\$:$ds:AVERAGE";
	 $i++;
	 my $t_uom = "";
	 if ( $perfdata->{$l}->{uom} and $perfdata->{$l}->{uom} !~ /c/i ) {
	    $t_uom = $perfdata->{$l}->{uom};
	    $t_uom =~ s/%/%%/g;    # for values marked as percent
	    $t_uom = "%s$t_uom";
	 }
	 push @rrdtool_options, "GPRINT:$ds:LAST:\"%8.2lf $t_uom\"";
	 push @rrdtool_options, "GPRINT:$ds:AVERAGE:\"%8.2lf $t_uom\"";
	 push @rrdtool_options, "GPRINT:$ds:MAX:\"%8.2lf $t_uom\"";
	 push @rrdtool_options,
	   "#use color $colors[$i] -> $colors{$colors[$i]}";
	 push @rrdtool_options, "LINE2:$ds$colors{$colors[$i]}:$l\:";
	 push @rrdtool_options, "COMMENT:\"\\n\"";
	 print "ERROR: Not enough color available ($i/$max_colors) \n"
	   if $i > $max_colors;
      }
   }
}

sub write_graph_template_to_file($) {
   my $ofile = shift;
   open TEMPLATE, '>', "$ofile"
     or log_msg("Can't open file \"$ofile\" for writing\n")
     and return 2;
   log_msg("Creating Graph template \"$ofile\"\n") if $debug;
   foreach my $line (@rrdtool_options) {
      print TEMPLATE "$line\n";
   }
   close TEMPLATE;

   # just in case it's reused
   @rrdtool_options = ();
   return 0;
}

sub load_rgb_colors($) {
   my $rgb_file = shift;

   open COLORS, '<', "$rgb_file" or die "Can't open rgb colors file";

   while (<COLORS>) {
      next if /^#/;    #skip comments
      next if /^$/;    #skip empty lines
      chomp;

      my ( $name, $hex ) = split /=/, $_, 2;

      #print "DEBUG: $name -> $hex\n";
      $colors{$name} = $hex;
      push @colors, $name;
   }

   close COLORS;
}

#
# funtion that parses performance data
# the result is stored in global ref $perfdata
sub parse_perfdata($) {
   my $perf_str = shift;

   # empty array
   $perfdata = {};

   my @tmp_data = ();

# we want the actual dsnames, sometimes RRD may contain DS which is not updated anymore
   @dsnames = ();
   my @vals = (
      $perf_str =~ /([^=]+	  # var name
				=[-+]?[0-9\.,]+       # value mandatory
				[\w%]*[ ;]?         # uom
				[-+]?[0-9\.,]*;?     # warning
				[-+]?[0-9\.,]*;?     # critical
				[-+]?[0-9\.,]*;?     # min
				[-+]?[0-9\.,]*;?\s*) # max
			/xsmg
   );

   foreach my $v (@vals) {

      #print "DEBUG: $v\n";
      my ( $label, $val, $uom, $w, $c, $min, $max ) = (
	 $v =~ /([^=]+)	           # var name
				=([-+]?[0-9\.,]+)      # value mandatory
				([\w%]*)[ ;]?        # uom
				([-+]?[0-9\.,]*);?    # warning
				([-+]?[0-9\.,]*);?    # critical
				([-+]?[0-9\.,]*);?    # min
				([-+]?[0-9\.,]*);?\s* # max
				/xg
      );

      $uom = $uom;
      $uom = uc $uom;

      $w   = $w   || 0;
      $c   = $c   || 0;
      $min = $min || 0;
      $max = $max || 0;

  #
  # I assume only "," or "." are part of numbers
  # in my case I found that if system LANG is set to "DE*", the output format of
  # some plugins are affected, which in turn breakes rrd data update
  #
      $val =~ s/,/./g if $val;
      $w   =~ s/,/./g if $w;
      $c   =~ s/,/./g if $c;
      $min =~ s/,/./g if $min;
      $max =~ s/,/./g if $max;

      $perfdata->{$label}->{value}    = $val;
      $perfdata->{$label}->{uom}      = $uom;
      $perfdata->{$label}->{warning}  = $w;
      $perfdata->{$label}->{critical} = $c;
      $perfdata->{$label}->{min}      = $min;
      $perfdata->{$label}->{max}      = $max;

# Convert only if enabled
# value, warning, critical, min, max are all UOM based values as per Nagios documents
      my @values = qw(
	value
	warning
	critical
	min
	max
      );

      if ( defined $fact_bytes->{ $perfdata->{$label}->{uom} }
	 and $config{DYN_DS_CONVERT_TO_BYTES} )
      {
	 my @t = map {
	    $perfdata->{$label}->{$_} =
	      $perfdata->{$label}->{$_} *
	      $fact_bytes->{ $perfdata->{$label}->{uom} }->{val}
	 } @values;
	 $perfdata->{$label}->{uom} =
	   $fact_bytes->{ $perfdata->{$label}->{uom} }->{result};
      }
      elsif ( defined $fact_seconds->{ $perfdata->{$label}->{uom} }
	 and $config{DYN_DS_CONVERT_TO_SECONDS} )
      {
	 my @t = map {
	    $perfdata->{$label}->{$_} =
	      $perfdata->{$label}->{$_} *
	      $fact_seconds->{ $perfdata->{$label}->{uom} }->{val}
	 } @values;
	 $perfdata->{$label}->{uom} =
	   $fact_seconds->{ $perfdata->{$label}->{uom} }->{result};
      }

      if ( $label eq '/' ) {
	 $perfdata->{$label}->{clean} = "root" if $label eq '/';
      }
      else {
	 my $temp_label = $label;

	 #
	 # rrd DS names are limited to 19 characters
	 # and only alphanumeric plus "_" is allowed
	 $temp_label =~ s/'//g;# remove
	 $temp_label =~ s/[^\w]/_/g;
	 $temp_label = substr( $temp_label, 0, 19 )
	   if length($temp_label) > 19;

	 $perfdata->{$label}->{clean} = $temp_label;
      }
      push @tmp_data,
	"$perfdata->{$label}->{clean}=$perfdata->{$label}->{value}";
      push @dsnames, $perfdata->{$label}->{clean};
   }
   return @tmp_data;
}

sub get_check_interval($$$$) {
   my ( $hostname, $o_type, $service_description, $status_file ) = @_;
   my $match_found = 0;
   $object = {};
   $object_end   = undef;
   open STATUS, "<$status_file"
     or die "can't open status file \"$status_file\"";
   while ( my $line = <STATUS> ) {
      if ( $line =~ /\s*(info|service|servicestatus|host|hoststatus)\s*{.*$/ ) {
	 $object_start = $1;
	 $object_start = "service" if $1 =~ /^service/;
      }
      elsif ( $object_start && $line =~ /\s*(\w+)=(.*)$/ ) {
	 $object->{$object_start}->{$1} = "$2";
      }
      elsif ( $object_start && $line =~ /\s*}$/ ) {
	 $object_end++;
      }
      else { next; }

      #
      # parsed one object and its value
      if ($object_end) {
	 if ( $object_start eq 'info' ) {
	    $nagios_version = $object->{$object_start}->{version}
	      if $object_start =~ /info/;
	    $nagios_version =~ s/(\d+).*/$1/;
	    return 0 if $nagios_version < 2;

	    next;
	 }

	 if (  ( defined $object->{$object_start}->{host_name} )
	    && ( $object->{$object_start}->{host_name} =~ /$hostname/ ) )
	 {
	    my $check_interval = 0;
	    $check_interval = int(
	       (
		  $object->{$object_start}->{next_check} -
		    $object->{$object_start}->{last_check}
	       ) / 60
	    ) if $nagios_version == 2;

	    if ( $nagios_version == 3 )
	    { # between 3.0 and 3.1 variable names are different, but the values are from same source
	       $check_interval =
		 int( $object->{$object_start}->{check_interval} )
		 if defined $object->{$object_start}->{check_interval};
	       $check_interval =
		 int( $object->{$object_start}->{normal_check_interval} )
		 if defined $object->{$object_start}->{normal_check_interval};
	    }

	    if ( $object_start =~ /hoststatus/ && $o_type =~ /host/i ) {
	       return "$check_interval";
	    }
	    elsif ( $object_start =~ /service/ ) {
	       $match_found++
		 if $object->{$object_start}->{service_description} =~
		    /$service_description/
		    and $object->{$object_start}->{host_name} =~ /$hostname/;
	       return "$check_interval" if $match_found;
	    }
	 }
      }

      if ($object_end) {
	 $object_start = undef;
	 $object_end   = undef;
	 $match_found  = 0;
	 $object       = {};
      }
   }
   return 0;
}

exit 0;
